/*------------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG, 1990-2016

   Project:  Eloquence DLL interface
   Module:   dllif
   File:     dllif.h
   Function: DLL interface header file

   Note: This file should NOT be included when processing
   a source with dllcc

   Edit history:
   22.11.1990 - creation
   20.12.1992 - K&R compliance
   20.01.1995 - MAX_ARGUMENT 20 -> 50
   21.02.2003 - LP64 clean
   08.09.2016 - B.08.30, _DLLIF_PROTOTYPES, DLLIF_SHARED

------------------------------------------------------------------------*/

/* @(#)$Revision: 28.2 $ */

#ifndef _DLLIF_INCLUDED
#define _DLLIF_INCLUDED
#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------
   DLL interface limits
------------------------------------------------------------------------*/

#define MAX_ARGUMENT 50


/*------------------------------------------------------------------------
   DLL interface types
------------------------------------------------------------------------*/

typedef int EqDLL;

typedef void EqVoid;
typedef int EqInt;
typedef double EqReal;
typedef char EqChar;

struct EqString {
   int maxl;         /* maximum string length */
   int l;            /* current string length */
   char *p;          /* string value */
};

typedef struct EqString EqString;


/*------------------------------------------------------------------------
   DLL interface utility types
------------------------------------------------------------------------*/

enum EqArgType {
   EqArg_VarArg,
   EqArg_IntByValue,
   EqArg_IntByReference,
   EqArg_IntArray,
   EqArg_RealByValue,
   EqArg_RealByReference,
   EqArg_RealArray,
   EqArg_zString,
   EqArg_String,
   EqArg_StringArray
};

struct EqFuncInfo {
   char *name;
   int arg_cnt;
};

struct EqArgInfo {
   enum EqArgType type;
   char *name;
   int elcnt;
};

struct EqFuncList {
   char *name;
   int arg_idx;
   int arg_cnt;
   int call_arg_cnt;
};

struct EqArgList {
   char *name;
   enum EqArgType type;
   int size;
};


/*------------------------------------------------------------------------
   DLL interface global variables
   (Windows platform: imported from shared eqdll.dll)
------------------------------------------------------------------------*/

#ifdef _WIN32
#ifdef DLLIF_SHARED
#define EqDLL_IMPORT __declspec(dllimport)
#define EqDLL_EXPORT __declspec(dllexport)
#endif
#endif

#ifndef EqDLL_IMPORT
#define EqDLL_IMPORT /**/
#endif
#ifndef EqDLL_EXPORT
#define EqDLL_EXPORT /**/
#endif

/*
   DLL debug levels
    0 = no debugging (default)
    1 = function call and return value
    2 = function call, argument values, return value
    3 = internal
    4 = internal (inter-process communication)
*/

#ifdef _DLL_PRIVATE
extern int dll_debug;
#else
extern EqDLL_IMPORT int dll_debug;
#endif


/*------------------------------------------------------------------------
   DLL interface utility function prototypes
------------------------------------------------------------------------*/

#ifndef _DLLIF_PROTOTYPES
#ifdef __hpux
/*
   The K&R compiler does not support prototypes. While everyone should
   use the ANSI compiler for anything serious, it may not be available
   because it is sold extra. So we make prototypes optional for now.
*/
#ifdef __cplusplus
#define _DLLIF_PROTOTYPES 1
#else
#ifdef __STDC__
#define _DLLIF_PROTOTYPES 1
#endif

#endif
#else
/*
   Here goes everything that's not HP-UX. Since K&R is a HP-UX only
   problem and everyone would use a ANSI compliant compiler anyway,
   we simply enable function prototypes.
*/
#define _DLLIF_PROTOTYPES 1
#endif
#endif /* _DLLIF_PROTOTYPES */

#if _DLLIF_PROTOTYPES
#define _p(a) a
#else
#define _p(a) ()
#endif

struct EqFuncInfo *EqFuncInfo _p((void));
struct EqArgInfo *EqArgInfo _p((void *arg));
int EqStrlen _p((EqString *s));
int EqMaxStrlen _p((EqString *s));
int EqStrcat _p((EqString *s1, EqString *s2));
int EqStrcpy _p((EqString *s1, EqString *s2));
int EqStrcmp _p((EqString *s1, EqString *s2));
EqString *EqSubstr _p((EqString *s, int start, int len));
int EqStr2str _p((EqString *eq, void *s));
int str2EqStr _p((void *s, EqString *eq));
EqString *EqMkstr _p((int maxl));

#ifdef __cplusplus
}
#endif
#endif /* _DLLIF_INCLUDED */
